@@ -5,6 +5,7 @@ import (
 	"bytes"
 	"encoding/json"
 	"fmt"
+	"io/ioutil"
 	"os"
 	"os/exec"
 	"os/user"
@@ -25,6 +26,7 @@ import (
 	errors "github.com/pkg/errors"
 	"github.com/vincent-petithory/dataurl"
 	corev1 "k8s.io/api/core/v1"
+	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/util/wait"
 )
 
@@ -37,8 +39,8 @@ const (
 	coreUserName = "core"
 	// SSH Keys for user "core" will only be written at /home/core/.ssh
 	coreUserSSHPath = "/home/core/.ssh/"
-	// fipsCommand is the command to use when enabling or disabling FIPS
-	fipsCommand = "/usr/libexec/rhcos-tools/coreos-fips"
+	// fipsFile is the file to check if FIPS is enabled
+	fipsFile = "/proc/sys/crypto/fips_enabled"
 )
 
 func writeFileAtomicallyWithDefaults(fpath string, b []byte) error {
@@ -132,6 +134,7 @@ func (dn *Daemon) drain() error {
 	}
 
 	dn.logSystem("Update prepared; beginning drain")
+	startTime := time.Now()
 
 	dn.recorder.Eventf(getNodeRef(dn.node), corev1.EventTypeNormal, "Drain", "Draining node to update config.")
 
@@ -156,19 +159,56 @@ func (dn *Daemon) drain() error {
 		glog.Infof("Draining failed with: %v, retrying", err)
 		return false, nil
 	}); err != nil {
+		failTime := fmt.Sprintf("%v sec", time.Since(startTime).Seconds())
 		if err == wait.ErrWaitTimeout {
+			failMsg := fmt.Sprintf("%d tries: %v", backoff.Steps, lastErr)
+			MCDDrainErr.WithLabelValues(failTime, failMsg).SetToCurrentTime()
 			return errors.Wrapf(lastErr, "failed to drain node (%d tries): %v", backoff.Steps, err)
 		}
+		MCDDrainErr.WithLabelValues(failTime, err.Error()).SetToCurrentTime()
 		return errors.Wrap(err, "failed to drain node")
 	}
+
 	dn.logSystem("drain complete")
+	t := time.Since(startTime).Seconds()
+	glog.Infof("Successful drain took %v seconds", t)
+	successTime := fmt.Sprintf("%v sec", t)
+	MCDDrainErr.WithLabelValues(successTime, "").Set(0)
+
 	return nil
 }
 
 var errUnreconcilable = errors.New("unreconcilable")
 
+func canonicalizeEmptyMC(config *mcfgv1.MachineConfig) *mcfgv1.MachineConfig {
+	if config != nil {
+		return config
+	}
+	return &mcfgv1.MachineConfig{
+		ObjectMeta: metav1.ObjectMeta{Name: "mco-empty-mc"},
+		Spec: mcfgv1.MachineConfigSpec{
+			Config: ctrlcommon.NewIgnConfig(),
+		},
+	}
+}
+
+func (dn *Daemon) compareMachineConfig(oldConfig, newConfig *mcfgv1.MachineConfig) bool {
+	var diff *MachineConfigDiff
+	oldConfig = canonicalizeEmptyMC(oldConfig)
+	oldConfigName := oldConfig.GetName()
+	newConfigName := newConfig.GetName()
+	diff = NewMachineConfigDiff(oldConfig, newConfig)
+	if diff.IsEmpty() {
+		glog.Infof("No changes from %s to %s", oldConfigName, newConfigName)
+		return false
+	}
+	return true
+}
+
 // update the node to the provided node configuration.
 func (dn *Daemon) update(oldConfig, newConfig *mcfgv1.MachineConfig) (retErr error) {
+	oldConfig = canonicalizeEmptyMC(oldConfig)
+
 	if dn.nodeWriter != nil {
 		state, err := getNodeAnnotationExt(dn.node, constants.MachineConfigDaemonStateAnnotationKey, true)
 		if err != nil {
@@ -188,36 +228,29 @@ func (dn *Daemon) update(oldConfig, newConfig *mcfgv1.MachineConfig) (retErr err
 		}
 	}()
 
-	oldConfigUnset := oldConfig == nil
-	if oldConfigUnset {
-		// Rather than change the rest of the code to deal
-		// with a nil oldConfig, just pass an empty one in
-		// most places.
-		emptyMC := mcfgv1.MachineConfig{}
-		oldConfig = &emptyMC
-	} else {
-		oldConfigName := oldConfig.GetName()
-		newConfigName := newConfig.GetName()
-		glog.Infof("Checking Reconcilable for config %v to %v", oldConfigName, newConfigName)
+	oldConfigName := oldConfig.GetName()
+	newConfigName := newConfig.GetName()
 
-		// make sure we can actually reconcile this state
-		diff, reconcilableError := Reconcilable(oldConfig, newConfig)
+	glog.Infof("Checking Reconcilable for config %v to %v", oldConfigName, newConfigName)
 
-		if reconcilableError != nil {
-			wrappedErr := fmt.Errorf("can't reconcile config %s with %s: %v", oldConfigName, newConfigName, reconcilableError)
-			if dn.recorder != nil {
-				mcRef := &corev1.ObjectReference{
-					Kind: "MachineConfig",
-					Name: newConfig.GetName(),
-					UID:  newConfig.GetUID(),
-				}
-				dn.recorder.Eventf(mcRef, corev1.EventTypeWarning, "FailedToReconcile", wrappedErr.Error())
+	// make sure we can actually reconcile this state
+	diff, reconcilableError := Reconcilable(oldConfig, newConfig)
+
+	if reconcilableError != nil {
+		wrappedErr := fmt.Errorf("can't reconcile config %s with %s: %v", oldConfigName, newConfigName, reconcilableError)
+		if dn.recorder != nil {
+			mcRef := &corev1.ObjectReference{
+				Kind: "MachineConfig",
+				Name: newConfig.GetName(),
+				UID:  newConfig.GetUID(),
 			}
-			return errors.Wrapf(errUnreconcilable, "%v", wrappedErr)
+			dn.recorder.Eventf(mcRef, corev1.EventTypeWarning, "FailedToReconcile", wrappedErr.Error())
 		}
-		dn.logSystem("Starting update from %s to %s: %+v", oldConfigName, newConfigName, diff)
+		return errors.Wrapf(errUnreconcilable, "%v", wrappedErr)
 	}
 
+	dn.logSystem("Starting update from %s to %s: %+v", oldConfigName, newConfigName, diff)
+
 	if err := dn.drain(); err != nil {
 		return err
 	}
@@ -253,7 +286,7 @@ func (dn *Daemon) update(oldConfig, newConfig *mcfgv1.MachineConfig) (retErr err
 		return err
 	}
 	defer func() {
-		if retErr != nil && !oldConfigUnset {
+		if retErr != nil {
 			if err := dn.storeCurrentConfigOnDisk(oldConfig); err != nil {
 				retErr = errors.Wrapf(retErr, "error rolling back current config on disk %v", err)
 				return
@@ -274,40 +307,9 @@ func (dn *Daemon) update(oldConfig, newConfig *mcfgv1.MachineConfig) (retErr err
 		}
 	}()
 
-	if err := dn.updateFIPS(oldConfig, newConfig); err != nil {
-		return err
-	}
-	defer func() {
-		if retErr != nil {
-			if err := dn.updateFIPS(newConfig, oldConfig); err != nil {
-				retErr = errors.Wrapf(retErr, "error rolling back FIPS %v", err)
-				return
-			}
-		}
-	}()
-
 	return dn.updateOSAndReboot(newConfig)
 }
 
-func (dn *Daemon) updateFIPS(current, desired *mcfgv1.MachineConfig) error {
-	if current.Spec.Fips == desired.Spec.Fips {
-		return nil
-	}
-	if dn.OperatingSystem != machineConfigDaemonOSRHCOS {
-		return errors.New("Updating FIPS on non-RHCOS nodes is not supported")
-	}
-	arg := "enable"
-	if !desired.Spec.Fips {
-		arg = "disable"
-	}
-	cmd := exec.Command(fipsCommand, arg)
-	dn.logSystem("Running %s %s", fipsCommand, arg)
-	if out, err := cmd.CombinedOutput(); err != nil {
-		return errors.Wrapf(err, "%s FIPS: %s", arg, string(out))
-	}
-	return nil
-}
-
 // MachineConfigDiff represents an ad-hoc difference between two MachineConfig objects.
 // At some point this may change into holding just the files/units that changed
 // and the MCO would just operate on that.  For now we're just doing this to get
@@ -321,6 +323,35 @@ type MachineConfigDiff struct {
 	units    bool
 }
 
+// NewMachineConfigDiff compares two MachineConfig objects.
+func NewMachineConfigDiff(oldConfig, newConfig *mcfgv1.MachineConfig) *MachineConfigDiff {
+	oldIgn := oldConfig.Spec.Config
+	newIgn := newConfig.Spec.Config
+
+	// Both nil and empty slices are of zero length,
+	// consider them as equal while comparing KernelArguments in both MachineConfigs
+	kargsEmpty := len(oldConfig.Spec.KernelArguments) == 0 && len(newConfig.Spec.KernelArguments) == 0
+
+	return &MachineConfigDiff{
+		osUpdate: oldConfig.Spec.OSImageURL != newConfig.Spec.OSImageURL,
+		kargs:    !(kargsEmpty || reflect.DeepEqual(oldConfig.Spec.KernelArguments, newConfig.Spec.KernelArguments)),
+		fips:     oldConfig.Spec.FIPS != newConfig.Spec.FIPS,
+		passwd:   !reflect.DeepEqual(oldIgn.Passwd, newIgn.Passwd),
+		files:    !reflect.DeepEqual(oldIgn.Storage.Files, newIgn.Storage.Files),
+		units:    !reflect.DeepEqual(oldIgn.Systemd.Units, newIgn.Systemd.Units),
+	}
+}
+
+// IsEmpty returns true if the MachineConfigDiff has no changes, or
+// in other words if the two MachineConfig objects are equivalent from
+// the MCD's point of view.  This is mainly relevant if e.g. two MC
+// objects happen to have different Ignition versions but are otherwise
+// the same.  (Probably a better way would be to canonicalize)
+func (d *MachineConfigDiff) IsEmpty() bool {
+	emptyDiff := MachineConfigDiff{}
+	return reflect.DeepEqual(d, &emptyDiff)
+}
+
 // Reconcilable checks the configs to make sure that the only changes requested
 // are ones we know how to do in-place.  If we can reconcile, (nil, nil) is returned.
 // Otherwise, if we can't do it in place, the node is marked as degraded;
@@ -424,16 +455,15 @@ func Reconcilable(oldConfig, newConfig *mcfgv1.MachineConfig) (*MachineConfigDif
 
 	// we can reconcile any state changes in the systemd section.
 
+	// FIPS section
+	// We do not allow update to FIPS for a running cluster, so any changes here will be an error
+	if err := checkFIPS(oldConfig, newConfig); err != nil {
+		return nil, err
+	}
+
 	// we made it through all the checks. reconcile away!
 	glog.V(2).Info("Configs are reconcilable")
-	return &MachineConfigDiff{
-		osUpdate: oldConfig.Spec.OSImageURL != newConfig.Spec.OSImageURL,
-		kargs:    !reflect.DeepEqual(oldConfig.Spec.KernelArguments, newConfig.Spec.KernelArguments),
-		fips:     oldConfig.Spec.Fips != newConfig.Spec.Fips,
-		passwd:   passwdChanged,
-		files:    !reflect.DeepEqual(oldIgn.Storage.Files, newIgn.Storage.Files),
-		units:    !reflect.DeepEqual(oldIgn.Systemd.Units, newIgn.Systemd.Units),
-	}, nil
+	return NewMachineConfigDiff(oldConfig, newConfig), nil
 }
 
 // verifyUserFields returns nil if the user Name = "core", if 1 or more SSHKeys exist for
@@ -457,6 +487,32 @@ func verifyUserFields(pwdUser igntypes.PasswdUser) error {
 	return nil
 }
 
+// checkFIPS verifies the state of FIPS on the system before an update.
+// Our new thought around this is that really FIPS should be a "day 1"
+// operation, and we don't want to make it editable after the fact.
+// See also https://github.com/openshift/installer/pull/2594
+// Anyone who wants to force this can change the MC flag, then
+// `oc debug node` and run the disable command by hand, then reboot.
+// If we detect that FIPS has been changed, we reject the update.
+func checkFIPS(current, desired *mcfgv1.MachineConfig) error {
+
+	content, err := ioutil.ReadFile(fipsFile)
+	if err != nil {
+		return errors.Wrapf(err, "Error reading FIPS file at %s: %s", fipsFile, string(content))
+	}
+	nodeFIPS, err := strconv.ParseBool(strings.TrimSuffix(string(content), "\n"))
+	if err != nil {
+		return errors.Wrapf(err, "Error parsing FIPS file at %s", fipsFile)
+	}
+	if desired.Spec.FIPS == nodeFIPS {
+		// Check if FIPS on the system is at the desired setting
+		current.Spec.FIPS = nodeFIPS
+		return nil
+	}
+
+	return errors.New("detected change to FIPS flag. Refusing to modify FIPS on a running cluster")
+}
+
 // generateKargsCommand performs a diff between the old/new MC kernelArguments,
 // and generates the command line arguments suitable for `rpm-ostree kargs`.
 // Note what we really should be doing though is also looking at the *current*
@@ -871,6 +927,7 @@ func (dn *Daemon) updateOS(config *mcfgv1.MachineConfig) error {
 
 	glog.Infof("Updating OS to %s", newURL)
 	if err := dn.NodeUpdaterClient.RunPivot(newURL); err != nil {
+		MCDPivotErr.WithLabelValues(newURL, err.Error()).SetToCurrentTime()
 		return fmt.Errorf("failed to run pivot: %v", err)
 	}
 
@@ -880,7 +937,7 @@ func (dn *Daemon) updateOS(config *mcfgv1.MachineConfig) error {
 func (dn *Daemon) getPendingStateLegacyLogger() (*journalMsg, error) {
 	glog.Info("logger doesn't support --jounald, grepping the journal")
 
-	cmdLiteral := "journalctl -o cat _UID=0 | grep -v audit | grep OPENSHIFT_MACHINE_CONFIG_DAEMON_LEGACY_LOG_HACK"
+	cmdLiteral := "journalctl -o cat _UID=0 | grep OPENSHIFT_MACHINE_CONFIG_DAEMON_LEGACY_LOG_HACK"
 	cmd := exec.Command("bash", "-c", cmdLiteral)
 	var combinedOutput bytes.Buffer
 	cmd.Stdout = &combinedOutput
@@ -1050,11 +1107,13 @@ func (dn *Daemon) reboot(rationale string) error {
 	// either, we just have one for the MCD itself.
 	if err := rebootCmd.Run(); err != nil {
 		dn.logSystem("failed to run reboot: %v", err)
+		MCDRebootErr.WithLabelValues("failed to run reboot", err.Error()).SetToCurrentTime()
 	}
 
 	// wait to be killed via SIGTERM from the kubelet shutting down
 	time.Sleep(defaultRebootTimeout)
 
 	// if everything went well, this should be unreachable.
+	MCDRebootErr.WithLabelValues("reboot failed", "this error should be unreachable, something is seriously wrong").SetToCurrentTime()
 	return fmt.Errorf("reboot failed; this error should be unreachable, something is seriously wrong")
 }
