@@ -389,6 +389,22 @@ func (optr *Operator) syncMachineConfigDaemon(config *renderConfig) error {
 		return err
 	}
 
+	// Only generate a new proxy cookie secret if the secret does not exist or if it has been deleted.
+	_, err = optr.kubeClient.CoreV1().Secrets(config.TargetNamespace).Get("cookie-secret", metav1.GetOptions{})
+	if apierrors.IsNotFound(err) {
+		cookieSecretBytes, err := renderAsset(config, "manifests/machineconfigdaemon/cookie-secret.yaml")
+		if err != nil {
+			return err
+		}
+		cookieSecret := resourceread.ReadSecretV1OrDie(cookieSecretBytes)
+		_, _, err = resourceapply.ApplySecret(optr.kubeClient.CoreV1(), cookieSecret)
+		if err != nil {
+			return err
+		}
+	} else if err != nil {
+		return err
+	}
+
 	mcdBytes, err := renderAsset(config, "manifests/machineconfigdaemon/daemonset.yaml")
 	if err != nil {
 		return err
@@ -636,7 +652,7 @@ func (optr *Operator) waitForDaemonsetRollout(resource *appsv1.DaemonSet) error
 		if d.Generation <= d.Status.ObservedGeneration && d.Status.UpdatedNumberScheduled == d.Status.DesiredNumberScheduled && d.Status.NumberUnavailable == 0 {
 			return true, nil
 		}
-		lastErr = fmt.Errorf("Daemonset %s is not ready. status: (desired: %d, updated: %d, ready: %d, unavailable: %d)", d.Name, d.Status.DesiredNumberScheduled, d.Status.UpdatedNumberScheduled, d.Status.NumberReady, d.Status.NumberAvailable)
+		lastErr = fmt.Errorf("Daemonset %s is not ready. status: (desired: %d, updated: %d, ready: %d, unavailable: %d)", d.Name, d.Status.DesiredNumberScheduled, d.Status.UpdatedNumberScheduled, d.Status.NumberReady, d.Status.NumberUnavailable)
 		return false, nil
 	}); err != nil {
 		if err == wait.ErrWaitTimeout {
